import greenfoot.*;

import java.util.List;
import java.util.ArrayList;

/**
 * Animal. This is the base class for all animals. In addition to the standard Actor
 * methods, it provides the ability to move and turn.
 * 
 * @author Michael Kolling
 * @author Mihaela Sabin
 * @version Sep 10, 2010
 *    - renamed original method 'eat' to 'remove' to eliminate confusion with the 
 *    Cat's method 'eat'. 
 *    - used in 'friendly-cat' scenarios as parent for Cat and MeanAnimal.
 * @version OCt 4, 2011
 *    - Updated block comments to be the same as block comments in little-crab-start.
 */
public class Animal extends Actor
{
    ////////////////// Constants ////////////////////
    private static final double WALKING_SPEED = 3.0;
    ////////////////// Constuctors ///////////////////
    
    /////////////////// Methods /////////////////////  
   
   /**
     * Tests if this animal overlaps with another object of given class.
     * @param clss The class of the object that might be at the location this animal is at.
     * @return True if this animal overlaps with another object of given class.
     * False if there is no such object at the same location.
     */
    public boolean canSee(Class clss)
    {
        Actor actor = getOneObjectAtOffset(0, 0, clss);
        return actor != null;        
    }

    
        /**
     * Tests if this animal is close to one of the edges of the world. 
     * @return True if at the edge, false otherwise.
     * Note that 20 pixels is the margin for "touching" an edge. 
     */
    public boolean atWorldEdge()
    {
        //If current x is too close to 0 or too close to world's width
        if(getX() < 20 || getX() > getWorld().getWidth() - 20) {
            return true; //return value is true and method execution stops here.
        }
        //If condition above was false 
        //AND current y is too close to 0 or too close to world's height
        if(getY() < 20 || getY() > getWorld().getHeight() - 20) {
            return true; //return value is true and method execution stops here.
        }

        //None of the conditions above was true. 
        return false;//return value is false and method execution stops here.
    }

    /**
     * Move forward in the current direction.
     */
    public void move()
    {
        double angle = Math.toRadians( getRotation() );
        int x = (int) Math.round(getX() + Math.cos(angle) * WALKING_SPEED);
        int y = (int) Math.round(getY() + Math.sin(angle) * WALKING_SPEED);
        
        setLocation(x, y);
    }

    /**
     * Try to eat an object of class 'clss'. This is only successful if there
     * is such an object where we currently are. Otherwise this method does
     * nothing.
     */
    public void eat(Class clss)
    {
        Actor actor = getOneObjectAtOffset(0, 0, clss);
        if(actor != null) {
            getWorld().removeObject(actor);
        }
    }
}
