import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Defines cats. 
 * 
 * @author Meagan Wolf
 * @version Sep 10, 2010
 */
public class Cat extends Animal
{

   
  
    /////////////////////// Instance variables ////////////////////////
    private boolean sleepy = false;
    private boolean hungry = false;
    private boolean bored = true;
     
    private GreenfootImage catImg;
    
   //////////////////////// Constructors ////////////////////////////////
    /**
     * Creates a cat and initialize its images
     */
    public Cat( ) 
    {
        catImg = new GreenfootImage("cat.png");
        setImage(catImg);
        Greenfoot.setSpeed(40);
    }
    
    
     /**
     * Acts this cat's behavior.
     * Called ONCE when Act button is pressed.
     * Called over and over again when Run button is pressed.
     */
    public void act()
    {
        keyControlMove();
        isRatCornered();
    }

    ////////////////////////// Methods ///////////////////////////////
   /**
    * Check weather a control key on the keyboard has been pressed.
    * If it has react accordingly.  
    */
   public void keyControlMove()
   {
       if (Greenfoot.isKeyDown("left"))
       {
           walkLeft(2);
       }
       if (Greenfoot.isKeyDown("right"))
       {
           walkRight(2);
       }
       if (Greenfoot.isKeyDown("up"))
       {
           walkUp(2);
       }   
       if (Greenfoot.isKeyDown("down"))
       {
           walkDown(2);
       }   
   }
   
    /**
     * Walks a given distance into a given direction, using given images.
     * @param distance - how far to walk
     * @param direction - in which direction to go
     * @param img1 - image displayed during walking, in alternation with img2
     * @param img2 - image displayed during walking, in alternation with img1
     */
    private void walk(int distance, int direction, String img1, String img2)
    {
        for (int i = 0; i < distance; i++) 
        {
            setImage(img1);
            wait(2);
            setLocation(getX( ) + direction, getY());
            setImage(img2);
            wait(2);
            setLocation(getX() + direction, getY());
        }
    }
    
        /**
     * Jumps a given distance into a given direction, using given images.
     * @param distance - how far to jump
     * @param direction - in which direction to go 
     * @param img1 - image displayed during jumping, in alternation with img2
     * @param img2 - image displayed during jumping, in alternation with img1
     */
    private void jump(int distance, int direction, String img1, String img2)
    {
        for (int i = 0; i < distance; i++) 
        {
            setImage(img1);
            wait(2);
            setLocation(getX( ), getY() + direction);
            setImage(img2);
            wait(2);
            setLocation(getX( ), getY()+ direction);
        }
    }
   
   /**
    * Walks a bit to the left. 
    * @param distance - determines how far to walk.  Small numbers (1 to 10) are recommended.
    */ 
   public void walkLeft(int distance)
   {
       walk(distance, -5,"CatWalkingLeft.jpg", "CatWalkingLeft-2.jpg");
   } 
   
   /**
    * Walks a bit to the right.
    * @param distance - determines how far to walk.  Small numbers (1 to 10) are recommended.  
    */
   public void walkRight(int distance)
   {
       walk(distance, 5, "CatWalkingRight.jpg", "CatWalkingRight-2.jpg");
   } 
   
   /**
    * Walks up.  
    * @param distance - determines how far the cat will walk up.  Small numbers (1 to 10) are recommended.
    */
   public void walkUp(int distance)
   {
       jump(distance, -5, "CatWalkingRight.jpg", "CatWalkingRight-2.jpg");
   } 
   
   /**
    * Walks down.
    *@param distance - determines how far the cat will walk down.  Small numbers (1 to 10) are recommended.   
    */
   public void walkDown(int distance)
   {
       jump(distance, 5, "CatJumpingDown.jpg", "CatJumpingDown-2.jpg");
   }
   
   /**
     * Tests if this animal is close to one of the edges of the world. 
     * @return True if at the edge, false otherwise.
     * Note that 20 pixels is the margin for "touching" an edge. 
     */
    public boolean isRatCornered()
    {
        //If current x is too close to 0 or too close to world's width
        if(getX() < 40 && getY() < 40) {
           eat(Rat.class);
           return true; //return value is true and method execution stops here.
        
        }
        
         //If current x is too close to 0 or too close to world's width
        if(getX() < 40 && getY() > getWorld().getHeight() - 40) {
           eat(Rat.class);
            return true; //return value is true and method execution stops here.
          
        }
        
        //If condition above was false 
        //AND current y is too close to 0 or too close to world's height
       if(getY() < 40 && getX() > getWorld().getWidth() - 40) {
            eat(Rat.class); 
           return true; //return value is true and method execution stops here.
            
       }
        
        if(getY() > getWorld().getHeight() - 40 && getX() > getWorld().getWidth() - 40) {
            eat(Rat.class);
            return true; //return value is true and method execution stops here.
          
       }
        
        //None of the conditions above was true. 
       return false; //return value is false and method execution stops here.
   }
    
     /**
     * Eats an object of the given class if there is one at the location this animal is at.
     * @param - The class of the object that might be at the location this animal is at.
     */
    public void eat(Class clss)
    {
        Actor actor = getOneObjectAtOffset(0, 0, clss);
        if(actor != null) {
            getWorld().removeObject(actor);
            CatRatButterflyWorld crbWorld = (CatRatButterflyWorld) getWorld();
            RatCounter ratcount = crbWorld.getRatCounter();
            ratcount.ratEaten();
            Scoreboard scoreboard = crbWorld.getScoreboard();
            scoreboard.bumpScore(20);
            
            Greenfoot.playSound("hooray.wav");
            if (ratcount.areAllRatsEaten()){
                scoreboard.gameover();
                Greenfoot.stop();
        }
      }
    }  
  
    /**
     * Waits for a given time.
     * @param time - how much time this cat waits.
     */
    public void wait(int time)
    {
        // Greenfoot.delay(time);
    }
   
  

}
